@@ -1,7 +1,7 @@
 /*
  * wee-string.c - string functions
  *
- * Copyright (C) 2003-2013 Sebastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2014 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -54,6 +54,11 @@
 #include "../plugins/plugin.h"
 
 
+#define IS_OCTAL_DIGIT(c) ((c >= '0') && (c <= '7'))
+#define HEX2DEC(c) (((c >= 'a') && (c <= 'f')) ? c - 'a' + 10 :         \
+                    ((c >= 'A') && (c <= 'F')) ? c - 'A' + 10 :         \
+                    c - '0')
+
 typedef uint32_t string_shared_count_t;
 
 struct t_hashtable *string_hashtable_shared = NULL;
@@ -623,74 +628,157 @@ string_strip (const char *string, int left, int right, const char *chars)
 }
 
 /*
- * Converts hex chars (\x??) to value.
+ * Converts escaped chars to their value.
+ *
+ * Following escaped chars are supported:
+ *   \"         double quote
+ *   \\         backslash
+ *   \a         alert (BEL)
+ *   \b         backspace
+ *   \e         escape
+ *   \f         form feed
+ *   \n         new line
+ *   \r         carriage return
+ *   \t         horizontal tab
+ *   \v         vertical tab
+ *   \0ooo      char as octal value (ooo is 0 to 3 digits)
+ *   \xhh       char as hexadecimal value (hh is 1 to 2 digits)
+ *   \uhhhh     unicode char as hexadecimal value (hhhh is 1 to 4 digits)
+ *   \Uhhhhhhhh unicode char as hexadecimal value (hhhhhhhh is 1 to 8 digits)
  *
  * Note: result must be freed after use.
  */
 
 char *
-string_convert_hex_chars (const char *string)
+string_convert_escaped_chars (const char *string)
 {
-    char *output, hex_str[8], *error;
-    int pos_output;
-    long number;
+    const unsigned char *ptr_string;
+    char *output, utf_char[16];
+    int pos_output, i, length;
+    unsigned int value;
 
+    /* the output length is always <= to string length */
     output = malloc (strlen (string) + 1);
-    if (output)
+    if (!output)
+        return NULL;
+
+    pos_output = 0;
+    ptr_string = (const unsigned char *)string;
+    while (ptr_string && ptr_string[0])
     {
-        pos_output = 0;
-        while (string && string[0])
+        if (ptr_string[0] == '\\')
         {
-            if (string[0] == '\\')
+            ptr_string++;
+            switch (ptr_string[0])
             {
-                string++;
-                switch (string[0])
-                {
-                    case '\\':
-                        output[pos_output++] = '\\';
-                        string++;
-                        break;
-                    case 'x':
-                    case 'X':
-                        if (isxdigit ((unsigned char)string[1])
-                            && isxdigit ((unsigned char)string[2]))
+                case '"':  /* double quote */
+                    output[pos_output++] = '"';
+                    ptr_string++;
+                    break;
+                case '\\':  /* backslash */
+                    output[pos_output++] = '\\';
+                    ptr_string++;
+                    break;
+                case 'a':  /* alert */
+                    output[pos_output++] = 7;
+                    ptr_string++;
+                    break;
+                case 'b':  /* backspace */
+                    output[pos_output++] = 8;
+                    ptr_string++;
+                    break;
+                case 'e':  /* escape */
+                    output[pos_output++] = 27;
+                    ptr_string++;
+                    break;
+                case 'f':  /* form feed */
+                    output[pos_output++] = 12;
+                    ptr_string++;
+                    break;
+                case 'n':  /* new line */
+                    output[pos_output++] = 10;
+                    ptr_string++;
+                    break;
+                case 'r':  /* carriage return */
+                    output[pos_output++] = 13;
+                    ptr_string++;
+                    break;
+                case 't':  /* horizontal tab */
+                    output[pos_output++] = 9;
+                    ptr_string++;
+                    break;
+                case 'v':  /* vertical tab */
+                    output[pos_output++] = 11;
+                    ptr_string++;
+                    break;
+                case '0':  /* char as octal value (0 to 3 digits) */
+                    value = 0;
+                    for (i = 0; (i < 3) && IS_OCTAL_DIGIT(ptr_string[i + 1]); i++)
+                    {
+                        value = (value * 8) + (ptr_string[i + 1] - '0');
+                    }
+                    output[pos_output++] = value;
+                    ptr_string += 1 + i;
+                    break;
+                case 'x':  /* char as hexadecimal value (1 to 2 digits) */
+                case 'X':
+                    if (isxdigit (ptr_string[1]))
+                    {
+                        value = 0;
+                        for (i = 0; (i < 2) && isxdigit (ptr_string[i + 1]); i++)
                         {
-                            snprintf (hex_str, sizeof (hex_str),
-                                      "0x%c%c", string[1], string[2]);
-                            number = strtol (hex_str, &error, 16);
-                            if (error && !error[0])
-                            {
-                                output[pos_output++] = number;
-                                string += 3;
-                            }
-                            else
-                            {
-                                output[pos_output++] = '\\';
-                                output[pos_output++] = string[0];
-                                string++;
-                            }
+                            value = (value * 16) + HEX2DEC(ptr_string[i + 1]);
                         }
-                        else
+                        output[pos_output++] = value;
+                        ptr_string += 1 + i;
+                    }
+                    else
+                    {
+                        output[pos_output++] = ptr_string[0];
+                        ptr_string++;
+                    }
+                    break;
+                case 'u':  /* unicode char as hexadecimal (1 to 4 digits) */
+                case 'U':  /* unicode char as hexadecimal (1 to 8 digits) */
+                    if (isxdigit (ptr_string[1]))
+                    {
+                        value = 0;
+                        for (i = 0;
+                             (i < ((ptr_string[0] == 'u') ? 4 : 8))
+                                 && isxdigit (ptr_string[i + 1]);
+                             i++)
                         {
-                            output[pos_output++] = string[0];
-                            string++;
+                            value = (value * 16) + HEX2DEC(ptr_string[i + 1]);
                         }
-                        break;
-                    default:
-                        output[pos_output++] = '\\';
-                        output[pos_output++] = string[0];
-                        string++;
-                        break;
-                }
-            }
-            else
-            {
-                output[pos_output++] = string[0];
-                string++;
+                        utf8_int_string (value, utf_char);
+                        if (utf_char[0])
+                        {
+                            length = strlen (utf_char);
+                            memcpy (output + pos_output, utf_char, length);
+                            pos_output += length;
+                        }
+                        ptr_string += 1 + i;
+                    }
+                    else
+                    {
+                        output[pos_output++] = ptr_string[0];
+                        ptr_string++;
+                    }
+                    break;
+                default:
+                    output[pos_output++] = '\\';
+                    output[pos_output++] = ptr_string[0];
+                    ptr_string++;
+                    break;
             }
         }
-        output[pos_output] = '\0';
+        else
+        {
+            output[pos_output++] = ptr_string[0];
+            ptr_string++;
+        }
     }
+    output[pos_output] = '\0';
 
     return output;
 }
@@ -985,7 +1073,7 @@ string_has_highlight (const char *string, const char *highlight_words)
 
 /*
  * Checks if a string has a highlight using a compiled regular expression (any
- * match in string must be surrounded by word chars).
+ * match in string must be surrounded by delimiters).
  */
 
 int
@@ -1034,7 +1122,7 @@ string_has_highlight_regex_compiled (const char *string, regex_t *regex)
 
 /*
  * Checks if a string has a highlight using a regular expression (any match in
- * string must be surrounded by word chars).
+ * string must be surrounded by delimiters).
  */
 
 int
