@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2013 Sebastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2014 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -47,7 +47,7 @@ extern char *string_expand_home (const char *path);
 extern char *string_remove_quotes (const char *string, const char *quotes);
 extern char *string_strip (const char *string, int left, int right,
                            const char *chars);
-extern char *string_convert_hex_chars (const char *string);
+extern char *string_convert_escaped_chars (const char *string);
 extern char *string_mask_to_regex (const char *mask);
 extern const char *string_regex_flags (const char *regex, int default_flags,
                                        int *flags);
