@@ -1,7 +1,7 @@
 /*
  * gui-window.c - window functions (used by all GUI)
  *
- * Copyright (C) 2003-2013 Sebastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2014 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -1549,8 +1549,28 @@ void
 gui_window_search_start (struct t_gui_window *window)
 {
     window->buffer->text_search = GUI_TEXT_SEARCH_BACKWARD;
-    if (window->buffer->text_search_where == 0)
-        window->buffer->text_search_where = GUI_TEXT_SEARCH_IN_MESSAGE;
+    if ((window->buffer->text_search_where == 0)
+        ||  CONFIG_BOOLEAN(config_look_buffer_search_force_default))
+    {
+        /* set default search values */
+        window->buffer->text_search_exact = CONFIG_BOOLEAN(config_look_buffer_search_case_sensitive);
+        window->buffer->text_search_regex = CONFIG_BOOLEAN(config_look_buffer_search_regex);
+        switch (CONFIG_INTEGER(config_look_buffer_search_where))
+        {
+            case CONFIG_LOOK_BUFFER_SEARCH_PREFIX:
+                window->buffer->text_search_where = GUI_TEXT_SEARCH_IN_PREFIX;
+                break;
+            case CONFIG_LOOK_BUFFER_SEARCH_MESSAGE:
+                window->buffer->text_search_where = GUI_TEXT_SEARCH_IN_MESSAGE;
+                break;
+            case CONFIG_LOOK_BUFFER_SEARCH_PREFIX_MESSAGE:
+                window->buffer->text_search_where = GUI_TEXT_SEARCH_IN_PREFIX | GUI_TEXT_SEARCH_IN_MESSAGE;
+                break;
+            default:
+                window->buffer->text_search_where = GUI_TEXT_SEARCH_IN_MESSAGE;
+                break;
+        }
+    }
     window->buffer->text_search_found = 0;
     gui_input_search_compile_regex (window->buffer);
     if (window->buffer->text_search_input)
@@ -1647,14 +1667,14 @@ gui_window_zoom (struct t_gui_window *window)
     }
     else
     {
-        /* save layout and zoom on current window */
+        /* store layout and zoom on current window */
         ptr_layout = gui_layout_alloc (GUI_LAYOUT_ZOOM);
         if (ptr_layout)
         {
             gui_layout_add (ptr_layout);
             hook_signal_send ("window_zoom",
                               WEECHAT_HOOK_SIGNAL_POINTER, gui_current_window);
-            gui_layout_window_save (ptr_layout);
+            gui_layout_window_store (ptr_layout);
             gui_window_merge_all (window);
             hook_signal_send ("window_zoomed",
                               WEECHAT_HOOK_SIGNAL_POINTER, gui_current_window);
