@@ -1,7 +1,7 @@
 /*
  * irc-redirect.c - redirection of IRC command output
  *
- * Copyright (C) 2010-2013 Sebastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2010-2014 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -135,6 +135,17 @@ struct t_irc_redirect_pattern irc_redirect_patterns_default[] =
       NULL,
       NULL, NULL,
     },
+    { "monitor", 0, 0,
+      /*
+       * monitor: start: 732: list of monitored nicks
+       *           stop: 733: end of a monitor list
+       *          extra; -
+       */
+      "732:2",
+      "733:1",
+      NULL,
+      NULL, NULL,
+    },
     { "names", 0, 0,
       /*
        * names: start: 353: list of nicks on channel
