@@ -1,7 +1,7 @@
 /*
  * relay-command.c - relay command
  *
- * Copyright (C) 2003-2013 Sebastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2014 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -307,60 +307,58 @@ relay_command_relay (void *data, struct t_gui_buffer *buffer, int argc,
 void
 relay_command_init ()
 {
-    weechat_hook_command ("relay",
-                          N_("relay control"),
-                          N_("list|listfull|listrelay"
-                             " || add [ipv4.][ipv6.][ssl.]<protocol.name> <port>"
-                             " || del [ipv4.][ipv6.][ssl.]<protocol.name>"
-                             " || raw"
-                             " || sslcertkey"),
-                          N_("         list: list relay clients (only active "
-                             "relays)\n"
-                             "     listfull: list relay clients (verbose, all "
-                             "relays)\n"
-                             "    listrelay: list relays (name and port)\n"
-                             "          add: add relay for a protocol + name\n"
-                             "          del: remove relay for a protocol + name\n"
-                             "         ipv4: force use of IPv4\n"
-                             "         ipv6: force use of IPv6\n"
-                             "          ssl: enable SSL\n"
-                             "protocol.name: protocol and name to relay:\n"
-                             "                 - protocol \"irc\": name is the "
-                             "server to share (optional, if not given, the server "
-                             "name must be sent by client in command \"PASS\", "
-                             "with format: \"PASS server:password\")\n"
-                             "                 - protocol \"weechat\" (name is "
-                             "not used)\n"
-                             "         port: port used for relay\n"
-                             "          raw: open buffer with raw Relay data\n"
-                             "   sslcertkey: set SSL certificate/key using path "
-                             "in option relay.network.ssl_cert_key\n\n"
-                             "Without argument, this command opens buffer "
-                             "with list of relay clients.\n\n"
-                             "Examples:\n"
-                             "  irc proxy, for server \"freenode\":\n"
-                             "    /relay add irc.freenode 8000\n"
-                             "  irc proxy, for server \"freenode\", with SSL:\n"
-                             "    /relay add ssl.irc.freenode 8001\n"
-                             "  irc proxy, for all servers (client will choose), "
-                             "with SSL:\n"
-                             "    /relay add ssl.irc 8002\n"
-                             "  weechat protocol:\n"
-                             "    /relay add weechat 9000\n"
-                             "  weechat protocol with SSL:\n"
-                             "    /relay add ssl.weechat 9001\n"
-                             "  weechat protocol with SSL, using only IPv4:\n"
-                             "    /relay add ipv4.ssl.weechat 9001\n"
-                             "  weechat protocol with SSL, using only IPv6:\n"
-                             "    /relay add ipv6.ssl.weechat 9001\n"
-                             "  weechat protocol with SSL, using IPv4 + IPv6:\n"
-                             "    /relay add ipv4.ipv6.ssl.weechat 9001"),
-                          "list %(relay_relays)"
-                          " || listfull %(relay_relays)"
-                          " || listrelay"
-                          " || add %(relay_protocol_name) %(relay_free_port)"
-                          " || del %(relay_relays)"
-                          " || raw"
-                          " || sslcertkey",
-                          &relay_command_relay, NULL);
+    weechat_hook_command (
+        "relay",
+        N_("relay control"),
+        N_("list|listfull|listrelay"
+           " || add [ipv4.][ipv6.][ssl.]<protocol.name> <port>"
+           " || del [ipv4.][ipv6.][ssl.]<protocol.name>"
+           " || raw"
+           " || sslcertkey"),
+        N_("         list: list relay clients (only active relays)\n"
+           "     listfull: list relay clients (verbose, all relays)\n"
+           "    listrelay: list relays (name and port)\n"
+           "          add: add relay for a protocol + name\n"
+           "          del: remove relay for a protocol + name\n"
+           "         ipv4: force use of IPv4\n"
+           "         ipv6: force use of IPv6\n"
+           "          ssl: enable SSL\n"
+           "protocol.name: protocol and name to relay:\n"
+           "                 - protocol \"irc\": name is the server to share "
+           "(optional, if not given, the server name must be sent by client in "
+           "command \"PASS\", with format: \"PASS server:password\")\n"
+           "                 - protocol \"weechat\" (name is not used)\n"
+           "         port: port used for relay\n"
+           "          raw: open buffer with raw Relay data\n"
+           "   sslcertkey: set SSL certificate/key using path in option "
+           "relay.network.ssl_cert_key\n"
+           "\n"
+           "Without argument, this command opens buffer with list of relay "
+           "clients.\n"
+           "\n"
+           "Examples:\n"
+           "  irc proxy, for server \"freenode\":\n"
+           "    /relay add irc.freenode 8000\n"
+           "  irc proxy, for server \"freenode\", with SSL:\n"
+           "    /relay add ssl.irc.freenode 8001\n"
+           "  irc proxy, for all servers (client will choose), with SSL:\n"
+           "    /relay add ssl.irc 8002\n"
+           "  weechat protocol:\n"
+           "    /relay add weechat 9000\n"
+           "  weechat protocol with SSL:\n"
+           "    /relay add ssl.weechat 9001\n"
+           "  weechat protocol with SSL, using only IPv4:\n"
+           "    /relay add ipv4.ssl.weechat 9001\n"
+           "  weechat protocol with SSL, using only IPv6:\n"
+           "    /relay add ipv6.ssl.weechat 9001\n"
+           "  weechat protocol with SSL, using IPv4 + IPv6:\n"
+           "    /relay add ipv4.ipv6.ssl.weechat 9001"),
+        "list %(relay_relays)"
+        " || listfull %(relay_relays)"
+        " || listrelay"
+        " || add %(relay_protocol_name) %(relay_free_port)"
+        " || del %(relay_relays)"
+        " || raw"
+        " || sslcertkey",
+        &relay_command_relay, NULL);
 }
