@@ -2,7 +2,7 @@
  * weechat-tcl.c - tcl plugin for WeeChat
  *
  * Copyright (C) 2008-2010 Dmitry Kobylin <fnfal@academ.tsc.ru>
- * Copyright (C) 2008-2013 Sebastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2008-2014 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -688,6 +688,29 @@ weechat_tcl_signal_debug_dump_cb (void *data, const char *signal,
     return WEECHAT_RC_OK;
 }
 
+/*
+ * Display infos about external libraries used.
+ */
+
+int
+weechat_tcl_signal_debug_libs_cb (void *data, const char *signal,
+                                  const char *type_data, void *signal_data)
+{
+    /* make C compiler happy */
+    (void) data;
+    (void) signal;
+    (void) type_data;
+    (void) signal_data;
+
+#ifdef TCL_VERSION
+    weechat_printf (NULL, "  %s: %s", TCL_PLUGIN_NAME, TCL_VERSION);
+#else
+    weechat_printf (NULL, "  %s: (?)", TCL_PLUGIN_NAME);
+#endif
+
+    return WEECHAT_RC_OK;
+}
+
 /*
  * Callback called when a buffer is closed.
  */
@@ -806,6 +829,7 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
     init.callback_hdata = &weechat_tcl_hdata_cb;
     init.callback_infolist = &weechat_tcl_infolist_cb;
     init.callback_signal_debug_dump = &weechat_tcl_signal_debug_dump_cb;
+    init.callback_signal_debug_libs = &weechat_tcl_signal_debug_libs_cb;
     init.callback_signal_buffer_closed = &weechat_tcl_signal_buffer_closed_cb;
     init.callback_signal_script_action = &weechat_tcl_signal_script_action_cb;
     init.callback_load_file = &weechat_tcl_load_cb;
