@@ -1,7 +1,7 @@
 /*
  * xfer-command.c - xfer command
  *
- * Copyright (C) 2003-2013 Sebastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2014 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -157,15 +157,12 @@ xfer_command_xfer_list (int full)
                     weechat_printf (NULL,
                                     _("     plugin: %s (id: %s), file: %llu "
                                       "bytes (position: %llu), address: "
-                                      "%ld.%ld.%ld.%ld (port %d)"),
+                                      "%s (port %d)"),
                                     ptr_xfer->plugin_name,
                                     ptr_xfer->plugin_id,
                                     ptr_xfer->size,
                                     ptr_xfer->pos,
-                                    ptr_xfer->remote_address >> 24,
-                                    (ptr_xfer->remote_address >> 16) & 0xff,
-                                    (ptr_xfer->remote_address >> 8) & 0xff,
-                                    ptr_xfer->remote_address & 0xff,
+                                    ptr_xfer->remote_address_str,
                                     ptr_xfer->port);
                     date[0] = '\0';
                     date_tmp = localtime (&(ptr_xfer->start_transfer));
@@ -249,17 +246,19 @@ xfer_command_xfer (void *data, struct t_gui_buffer *buffer, int argc,
 void
 xfer_command_init ()
 {
-    weechat_hook_command ("me",
-                          N_("send a CTCP action to remote host"),
-                          N_("<message>"),
-                          N_("message: message to send"),
-                          NULL, &xfer_command_me, NULL);
-    weechat_hook_command ("xfer",
-                          N_("xfer control"),
-                          "[list|listfull]",
-                          N_("    list: list xfer\n"
-                             "listfull: list xfer (verbose)\n\n"
-                             "Without argument, this command opens buffer "
-                             "with xfer list."),
-                          "list|listfull", &xfer_command_xfer, NULL);
+    weechat_hook_command (
+        "me",
+        N_("send a CTCP action to remote host"),
+        N_("<message>"),
+        N_("message: message to send"),
+        NULL, &xfer_command_me, NULL);
+    weechat_hook_command (
+        "xfer",
+        N_("xfer control"),
+        "[list|listfull]",
+        N_("    list: list xfer\n"
+           "listfull: list xfer (verbose)\n"
+           "\n"
+           "Without argument, this command opens buffer with xfer list."),
+        "list|listfull", &xfer_command_xfer, NULL);
 }
