@@ -1,7 +1,7 @@
 /*
  * xfer.c - file transfer and direct chat plugin for WeeChat
  *
- * Copyright (C) 2003-2013 Sebastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2014 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -30,6 +30,8 @@
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <netdb.h>
+#include <gcrypt.h>
+#include <arpa/inet.h>
 
 #include "../weechat-plugin.h"
 #include "xfer.h"
@@ -45,25 +47,31 @@
 
 WEECHAT_PLUGIN_NAME(XFER_PLUGIN_NAME);
 WEECHAT_PLUGIN_DESCRIPTION(N_("DCC file transfer and direct chat"));
-WEECHAT_PLUGIN_AUTHOR("Sebastien Helleu <flashcode@flashtux.org>");
+WEECHAT_PLUGIN_AUTHOR("Sébastien Helleu <flashcode@flashtux.org>");
 WEECHAT_PLUGIN_VERSION(WEECHAT_VERSION);
 WEECHAT_PLUGIN_LICENSE(WEECHAT_LICENSE);
 
 struct t_weechat_plugin *weechat_xfer_plugin = NULL;
 
-char *xfer_type_string[] =                 /* strings for types             */
+char *xfer_type_string[] =             /* strings for types                 */
 { "file_recv", "file_send", "chat_recv",
   "chat_send"
 };
 
-char *xfer_protocol_string[] =             /* strings for protocols         */
+char *xfer_protocol_string[] =         /* strings for protocols             */
 { "none", "dcc"
 };
 
-char *xfer_status_string[] =               /* strings for status            */
+char *xfer_status_string[] =           /* strings for status                */
 { N_("waiting"), N_("connecting"),
   N_("active"), N_("done"), N_("failed"),
-  N_("aborted")
+  N_("aborted"), N_("hashing")
+};
+
+char *xfer_hash_status_string[] =      /* strings for hash status           */
+{ "?",
+  N_("CRC in progress"), N_("CRC OK"),
+  N_("wrong CRC"), N_("CRC error")
 };
 
 struct t_xfer *xfer_list = NULL;       /* list of files/chats               */
@@ -72,7 +80,6 @@ int xfer_count = 0;                    /* number of xfer                    */
 
 int xfer_signal_upgrade_received = 0;  /* signal "upgrade" received ?       */
 
-
 void xfer_disconnect_all ();
 
 
@@ -294,16 +301,13 @@ xfer_close (struct t_xfer *xfer, enum t_xfer_status status)
         if (XFER_IS_FILE(xfer->type))
         {
             weechat_printf (NULL,
-                            _("%s%s: file %s %s %s (%ld.%ld.%ld.%ld): %s"),
+                            _("%s%s: file %s %s %s (%s): %s"),
                             (xfer->status == XFER_STATUS_DONE) ? "" : weechat_prefix ("error"),
                             XFER_PLUGIN_NAME,
                             xfer->filename,
                             (xfer->type == XFER_TYPE_FILE_SEND) ? _("sent to") : _("received from"),
                             xfer->remote_nick,
-                            xfer->remote_address >> 24,
-                            (xfer->remote_address >> 16) & 0xff,
-                            (xfer->remote_address >> 8) & 0xff,
-                            xfer->remote_address & 0xff,
+                            xfer->remote_address_str,
                             (xfer->status == XFER_STATUS_DONE) ? _("OK") : _("FAILED"));
             xfer_network_child_kill (xfer);
         }
@@ -313,14 +317,12 @@ xfer_close (struct t_xfer *xfer, enum t_xfer_status status)
         if (XFER_IS_CHAT(xfer->type))
         {
             weechat_printf (xfer->buffer,
-                            _("%s: chat closed with %s "
-                              "(%ld.%ld.%ld.%ld)"),
+                            _("%s%s: chat closed with %s "
+                              "(%s)"),
+                            weechat_prefix ("network"),
                             XFER_PLUGIN_NAME,
                             xfer->remote_nick,
-                            xfer->remote_address >> 24,
-                            (xfer->remote_address >> 16) & 0xff,
-                            (xfer->remote_address >> 8) & 0xff,
-                            xfer->remote_address & 0xff);
+                            xfer->remote_address_str);
         }
     }
 
@@ -416,33 +418,12 @@ void
 xfer_send_signal (struct t_xfer *xfer, const char *signal)
 {
     struct t_infolist *infolist;
-    struct t_infolist_item *item;
-    char str_long[128];
 
     infolist = weechat_infolist_new ();
     if (infolist)
     {
-        item = weechat_infolist_new_item (infolist);
-        if (item)
+        if (xfer_add_to_infolist (infolist, xfer))
         {
-            weechat_infolist_new_var_string (item, "plugin_name", xfer->plugin_name);
-            weechat_infolist_new_var_string (item, "plugin_id", xfer->plugin_id);
-            weechat_infolist_new_var_string (item, "type", xfer_type_string[xfer->type]);
-            weechat_infolist_new_var_string (item, "protocol", xfer_protocol_string[xfer->protocol]);
-            weechat_infolist_new_var_string (item, "remote_nick", xfer->remote_nick);
-            weechat_infolist_new_var_string (item, "local_nick", xfer->local_nick);
-            weechat_infolist_new_var_string (item, "charset_modifier", xfer->charset_modifier);
-            weechat_infolist_new_var_string (item, "filename", xfer->filename);
-            snprintf (str_long, sizeof (str_long), "%llu", xfer->size);
-            weechat_infolist_new_var_string (item, "size", str_long);
-            snprintf (str_long, sizeof (str_long), "%llu", xfer->start_resume);
-            weechat_infolist_new_var_string (item, "start_resume", str_long);
-            snprintf (str_long, sizeof (str_long), "%lu", xfer->local_address);
-            weechat_infolist_new_var_string (item, "local_address", str_long);
-            snprintf (str_long, sizeof (str_long), "%lu", xfer->remote_address);
-            weechat_infolist_new_var_string (item, "remote_address", str_long);
-            weechat_infolist_new_var_integer (item, "port", xfer->port);
-
             weechat_hook_signal_send (signal, WEECHAT_HOOK_SIGNAL_POINTER,
                                       infolist);
         }
@@ -471,8 +452,12 @@ xfer_alloc ()
     /* default values */
     new_xfer->filename = NULL;
     new_xfer->size = 0;
-    new_xfer->local_address = 0;
-    new_xfer->remote_address = 0;
+    new_xfer->local_address = NULL;
+    new_xfer->local_address_length = 0;
+    new_xfer->local_address_str = NULL;
+    new_xfer->remote_address = NULL;
+    new_xfer->remote_address_length = 0;
+    new_xfer->remote_address_str = NULL;
     new_xfer->port = 0;
     new_xfer->remote_nick = NULL;
     new_xfer->local_nick = NULL;
@@ -566,6 +551,59 @@ xfer_nick_auto_accepted (const char *server, const char *nick)
     return rc;
 }
 
+/*
+ * Searches CRC32 in a filename.
+ *
+ * If more than one CRC32 are found, the last one is returned
+ * (with the higher index in filename).
+ *
+ * The chars before/after CRC32 must be either beginning/end of string or
+ * non-hexadecimal chars.
+ *
+ * Examples:
+ *
+ *   test_filename     => -1 (not found: no CRC32)
+ *   test_1234abcd     => 5  ("1234abcd")
+ *   1234abcd_test     => 0  ("1234abcd")
+ *   1234abcd_12345678 => 9  ("12345678")
+ *   123456789abcdef   => -1 (not found: missing delimiter around CRC32)
+ *
+ * Returns pointer to last CRC32 in string, NULL if no CRC32 was found.
+ */
+
+const char *
+xfer_filename_crc32 (const char *filename)
+{
+    int length;
+    const char *ptr_string, *ptr_crc32;
+
+    length = 0;
+    ptr_crc32 = NULL;
+
+    ptr_string = filename;
+    while (ptr_string && ptr_string[0])
+    {
+        if (((ptr_string[0] >= '0') && (ptr_string[0] <= '9'))
+            || ((ptr_string[0] >= 'A') && (ptr_string[0] <= 'F'))
+            || ((ptr_string[0] >= 'a') && (ptr_string[0] <= 'f')))
+        {
+            length++;
+        }
+        else
+        {
+            if (length == 8)
+                ptr_crc32 = ptr_string - 8;
+            length = 0;
+        }
+
+        ptr_string = weechat_utf8_next_char (ptr_string);
+    }
+    if (length == 8)
+        ptr_crc32 = ptr_string - 8;
+
+    return ptr_crc32;
+}
+
 /*
  * Adds a xfer to list.
  *
@@ -577,11 +615,14 @@ xfer_new (const char *plugin_name, const char *plugin_id,
           enum t_xfer_type type, enum t_xfer_protocol protocol,
           const char *remote_nick, const char *local_nick,
           const char *charset_modifier, const char *filename,
-          unsigned long long size, const char *proxy, unsigned long address,
-          int port, int sock, const char *local_filename)
+          unsigned long long size, const char *proxy, struct sockaddr *address,
+          socklen_t address_length, int port, int sock,
+          const char *local_filename)
 {
     struct t_xfer *new_xfer;
-    const char *ptr_color;
+    const char *ptr_color, *ptr_crc32;
+    char str_address[NI_MAXHOST];
+    int rc;
 
     new_xfer = xfer_alloc ();
     if (!new_xfer)
@@ -614,17 +655,36 @@ xfer_new (const char *plugin_name, const char *plugin_id,
         new_xfer->filename = strdup (_("xfer chat"));
     new_xfer->size = size;
     new_xfer->proxy = (proxy) ? strdup (proxy) : NULL;
+    new_xfer->port = port;
+
+    rc = getnameinfo ((struct sockaddr *)address, address_length, str_address,
+                      sizeof (str_address), NULL, 0, NI_NUMERICHOST);
+    if (rc != 0)
+    {
+        weechat_printf (NULL,
+                        _("%s%s: unable to interpret address: error %d %s"),
+                        weechat_prefix ("error"), XFER_PLUGIN_NAME,
+                        rc, gai_strerror (rc));
+        snprintf (str_address, sizeof (str_address), "?");
+    }
+
+    new_xfer->local_address = calloc (1, sizeof (struct sockaddr_storage));
+    new_xfer->local_address_length = sizeof (struct sockaddr_storage);
+    new_xfer->remote_address = calloc (1, sizeof (struct sockaddr_storage));
+    new_xfer->remote_address_length = sizeof (struct sockaddr_storage);
+
     if (XFER_IS_RECV(type))
     {
-        new_xfer->local_address = 0;
-        new_xfer->remote_address = address;
+        new_xfer->local_address_str = strdup ("");
+        new_xfer->remote_address_str = strdup (str_address);
+        memcpy (new_xfer->remote_address, address, address_length);
     }
     else
     {
-        new_xfer->local_address = address;
-        new_xfer->remote_address = 0;
+        new_xfer->local_address_str = strdup (str_address);
+        memcpy (new_xfer->local_address, address, address_length);
+        new_xfer->remote_address_str = strdup ("");
     }
-    new_xfer->port = port;
 
     new_xfer->status = XFER_STATUS_WAITING;
     new_xfer->sock = sock;
@@ -633,20 +693,47 @@ xfer_new (const char *plugin_name, const char *plugin_id,
     else
         xfer_file_find_filename (new_xfer);
 
+    new_xfer->hash_handle = NULL;
+    new_xfer->hash_target = NULL;
+    new_xfer->hash_status = XFER_HASH_STATUS_UNKNOWN;
+
+    if ((type == XFER_TYPE_FILE_RECV)
+        && weechat_config_boolean (xfer_config_file_auto_check_crc32))
+    {
+        ptr_crc32 = xfer_filename_crc32 (new_xfer->filename);
+        if (ptr_crc32)
+        {
+            new_xfer->hash_handle = malloc (sizeof (gcry_md_hd_t));
+            if (new_xfer->hash_handle)
+            {
+                if (gcry_md_open (new_xfer->hash_handle, GCRY_MD_CRC32, 0) == 0)
+                {
+                    new_xfer->hash_target = weechat_strndup (ptr_crc32, 8);
+                    new_xfer->hash_status = XFER_HASH_STATUS_IN_PROGRESS;
+                }
+                else
+                {
+                    free (new_xfer->hash_handle);
+                    new_xfer->hash_handle = NULL;
+                    weechat_printf (NULL,
+                                    _("%s%s: hashing error"),
+                                    weechat_prefix ("error"), XFER_PLUGIN_NAME);
+                }
+            }
+        }
+    }
+
     /* write info message on core buffer */
     switch (type)
     {
         case XFER_TYPE_FILE_RECV:
             weechat_printf (NULL,
                             _("%s: incoming file from %s "
-                              "(%ld.%ld.%ld.%ld, %s.%s), name: %s, %llu bytes "
+                              "(%s, %s.%s), name: %s, %llu bytes "
                               "(protocol: %s)"),
                             XFER_PLUGIN_NAME,
                             remote_nick,
-                            address >> 24,
-                            (address >> 16) & 0xff,
-                            (address >> 8) & 0xff,
-                            address & 0xff,
+                            str_address,
                             plugin_name,
                             plugin_id,
                             filename,
@@ -671,13 +758,10 @@ xfer_new (const char *plugin_name, const char *plugin_id,
         case XFER_TYPE_CHAT_RECV:
             weechat_printf (NULL,
                             _("%s: incoming chat request from %s "
-                              "(%ld.%ld.%ld.%ld, %s.%s)"),
+                              "(%s, %s.%s)"),
                             XFER_PLUGIN_NAME,
                             remote_nick,
-                            address >> 24,
-                            (address >> 16) & 0xff,
-                            (address >> 8) & 0xff,
-                            address & 0xff,
+                            str_address,
                             plugin_name,
                             plugin_id);
             xfer_buffer_refresh (WEECHAT_HOTLIST_MESSAGE);
@@ -745,6 +829,25 @@ xfer_new (const char *plugin_name, const char *plugin_id,
     return new_xfer;
 }
 
+/*
+ * Sets the remote address field.
+ */
+
+void
+xfer_set_remote_address (struct t_xfer *xfer, struct sockaddr *address,
+                         socklen_t length, char *address_str)
+{
+    if (xfer->remote_address)
+        free (xfer->remote_address);
+    xfer->remote_address = calloc (1, length);
+    xfer->remote_address_length = length;
+    memcpy (xfer->remote_address, address, length);
+
+    if (xfer->remote_address_str)
+        free (xfer->remote_address_str);
+    xfer->remote_address_str = strdup ((address_str) ? address_str : "");
+}
+
 /*
  * Frees xfer struct and removes it from list.
  */
@@ -781,12 +884,27 @@ xfer_free (struct t_xfer *xfer)
         free (xfer->charset_modifier);
     if (xfer->filename)
         free (xfer->filename);
+    if (xfer->local_address)
+        free (xfer->local_address);
+    if (xfer->local_address_str)
+        free (xfer->local_address_str);
+    if (xfer->remote_address)
+        free (xfer->remote_address);
+    if (xfer->remote_address_str)
+        free (xfer->remote_address_str);
     if (xfer->remote_nick_color)
         free (xfer->remote_nick_color);
     if (xfer->unterminated_message)
         free (xfer->unterminated_message);
     if (xfer->local_filename)
         free (xfer->local_filename);
+    if (xfer->hash_handle)
+    {
+        gcry_md_close (*xfer->hash_handle);
+        free (xfer->hash_handle);
+    }
+    if (xfer->hash_target)
+        free (xfer->hash_target);
 
     free (xfer);
 
@@ -808,15 +926,13 @@ xfer_add_cb (void *data, const char *signal, const char *type_data,
     struct t_infolist *infolist;
     const char *plugin_name, *plugin_id, *str_type, *str_protocol;
     const char *remote_nick, *local_nick, *charset_modifier, *filename, *proxy;
-    int type, protocol, args, port_start, port_end, sock, port;
-    const char *weechat_dir;
-    char *dir1, *dir2, *filename2, *short_filename, *pos;
+    const char *weechat_dir, *str_address, *str_port;
+    int type, protocol, args, port_start, port_end, sock, port, rc, own_ip_used;
+    char *dir1, *dir2, *filename2, *short_filename, *pos, str_port_temp[16];
     struct stat st;
-    struct hostent *host;
-    struct sockaddr_in addr;
+    struct addrinfo *ainfo, hints;
+    struct sockaddr_storage addr;
     socklen_t length;
-    struct in_addr tmpaddr;
-    unsigned long local_addr;
     unsigned long long file_size;
     struct t_xfer *ptr_xfer;
 
@@ -845,14 +961,15 @@ xfer_add_cb (void *data, const char *signal, const char *type_data,
 
     filename2 = NULL;
     short_filename = NULL;
+    file_size = 0;
 
     sock = -1;
     port = 0;
 
     plugin_name = weechat_infolist_string (infolist, "plugin_name");
     plugin_id = weechat_infolist_string (infolist, "plugin_id");
-    str_type = weechat_infolist_string (infolist, "type");
-    str_protocol = weechat_infolist_string (infolist, "protocol");
+    str_type = weechat_infolist_string (infolist, "type_string");
+    str_protocol = weechat_infolist_string (infolist, "protocol_string");
     remote_nick = weechat_infolist_string (infolist, "remote_nick");
     local_nick = weechat_infolist_string (infolist, "local_nick");
     charset_modifier = weechat_infolist_string (infolist, "charset_modifier");
@@ -898,10 +1015,6 @@ xfer_add_cb (void *data, const char *signal, const char *type_data,
         }
     }
 
-    filename2 = NULL;
-    file_size = 0;
-    port = 0;
-
     if (type == XFER_TYPE_FILE_RECV)
     {
         filename2 = strdup (filename);
@@ -953,10 +1066,8 @@ xfer_add_cb (void *data, const char *signal, const char *type_data,
             if (filename2[strlen (filename2) - 1] != DIR_SEPARATOR_CHAR)
                 strcat (filename2, DIR_SEPARATOR);
             strcat (filename2, filename);
-            if (dir1)
-                free (dir1);
-            if (dir2)
-                free (dir2);
+            free (dir1);
+            free (dir2);
         }
 #endif
         /* check if file exists */
@@ -966,66 +1077,75 @@ xfer_add_cb (void *data, const char *signal, const char *type_data,
                             _("%s%s: cannot access file \"%s\""),
                             weechat_prefix ("error"), XFER_PLUGIN_NAME,
                             filename2);
-            if (filename2)
-                free (filename2);
             goto error;
         }
         file_size = st.st_size;
     }
+    port = weechat_infolist_integer (infolist, "port");
+
+    /* set address */
+    memset (&hints, 0, sizeof (struct addrinfo));
+    hints.ai_flags = AI_NUMERICHOST | AI_NUMERICSERV;
+    hints.ai_family = AF_UNSPEC;  /* allow IPv4 or IPv6 */
+    hints.ai_socktype = SOCK_STREAM;
+    hints.ai_protocol = 0;
+    hints.ai_canonname = NULL;
+    hints.ai_addr = NULL;
+    hints.ai_next = NULL;
+
+    own_ip_used = 0;
 
     if (XFER_IS_RECV(type))
     {
-        sscanf (weechat_infolist_string (infolist, "remote_address"), "%lu", &local_addr);
-        port = weechat_infolist_integer (infolist, "port");
+        str_address = weechat_infolist_string (infolist, "remote_address");
+        snprintf (str_port_temp, sizeof (str_port_temp), "%d", port);
+        str_port = str_port_temp;
     }
     else
     {
-        /* get local IP address */
-        sscanf (weechat_infolist_string (infolist, "local_address"), "%lu", &local_addr);
-
-        memset (&addr, 0, sizeof (struct sockaddr_in));
-        addr.sin_family = AF_INET;
-        addr.sin_addr.s_addr = htonl (local_addr);
-
-        /* look up the IP address from network_own_ip, if set */
-        if (weechat_config_string(xfer_config_network_own_ip)
-            && weechat_config_string(xfer_config_network_own_ip)[0])
+        str_port = NULL;
+        if (weechat_config_string (xfer_config_network_own_ip)
+            && weechat_config_string (xfer_config_network_own_ip)[0])
         {
-            host = gethostbyname (weechat_config_string (xfer_config_network_own_ip));
-            if (host)
-            {
-                memcpy (&tmpaddr, host->h_addr_list[0], sizeof(struct in_addr));
-                local_addr = ntohl (tmpaddr.s_addr);
-
-                sock = weechat_infolist_integer (infolist, "socket");
-                if (sock > 0)
-                {
-                    memset (&addr, 0, sizeof (struct sockaddr_in));
-                    length = sizeof (addr);
-                    getsockname (sock, (struct sockaddr *) &addr, &length);
-                    addr.sin_family = AF_INET;
-                }
-            }
-            else
-            {
-                weechat_printf (NULL,
-                                _("%s%s: could not find address for \"%s\", "
-                                  "falling back to local IP"),
-                                weechat_prefix ("error"), XFER_PLUGIN_NAME,
-                                weechat_config_string (xfer_config_network_own_ip));
-            }
+            str_address = weechat_config_string (xfer_config_network_own_ip);
+            hints.ai_flags = AI_NUMERICSERV;
+            own_ip_used = 1;
         }
+        else
+            str_address = weechat_infolist_string (infolist, "local_address");
+    }
 
+    rc = getaddrinfo (str_address, str_port, &hints, &ainfo);
+    if ((rc == 0) && ainfo && ainfo->ai_addr)
+    {
+        memset (&addr,  0, sizeof (addr));
+        memcpy (&addr, ainfo->ai_addr, ainfo->ai_addrlen);
+        length = ainfo->ai_addrlen;
+        freeaddrinfo (ainfo);
+    }
+    else
+    {
+        weechat_printf (NULL,
+                        (own_ip_used) ?
+                        _("%s%s: invalid address \"%s\" (option "
+                          "xfer.network.own_ip): error %d %s") :
+                        _("%s%s: unable to find address for \"%s\": "
+                          "error %d %s"),
+                        weechat_prefix ("error"), XFER_PLUGIN_NAME,
+                        str_address, rc, gai_strerror (rc));
+        goto error;
+    }
+
+    if (XFER_IS_SEND(type))
+    {
         /* open socket for xfer */
-        sock = socket (AF_INET, SOCK_STREAM, 0);
+        sock = socket (addr.ss_family, SOCK_STREAM, 0);
         if (sock < 0)
         {
             weechat_printf (NULL,
                             _("%s%s: cannot create socket for xfer: error %d %s"),
                             weechat_prefix ("error"), XFER_PLUGIN_NAME,
                             errno, strerror (errno));
-            if (filename2)
-                free (filename2);
             goto error;
         }
 
@@ -1050,8 +1170,11 @@ xfer_add_cb (void *data, const char *signal, const char *type_data,
                     if (!xfer_port_in_use (port))
                     {
                         /* attempt to bind to the free port */
-                        addr.sin_port = htons (port);
-                        if (bind (sock, (struct sockaddr *) &addr, sizeof (addr)) == 0)
+                        if (addr.ss_family == AF_INET)
+                            ((struct sockaddr_in *)&addr)->sin_port = htons (port);
+                        else
+                            ((struct sockaddr_in6 *)&addr)->sin6_port = htons (port);
+                        if (bind (sock, (struct sockaddr *)&addr, length) == 0)
                             break;
                     }
                     port++;
@@ -1065,12 +1188,13 @@ xfer_add_cb (void *data, const char *signal, const char *type_data,
         if (port == 0)
         {
             /* find port automatically */
-            addr.sin_port = 0;
-            if (bind (sock, (struct sockaddr *) &addr, sizeof (addr)) == 0)
+            if (bind (sock, (struct sockaddr *)&addr, length) == 0)
             {
-                length = sizeof (addr);
-                getsockname (sock, (struct sockaddr *) &addr, &length);
-                port = ntohs (addr.sin_port);
+                getsockname (sock, (struct sockaddr *)&addr, &length);
+                if (addr.ss_family == AF_INET)
+                    port = ntohs (((struct sockaddr_in *)&addr)->sin_port);
+                else
+                    port = ntohs (((struct sockaddr_in6 *)&addr)->sin6_port);
             }
             else
                 port = -1;
@@ -1083,8 +1207,6 @@ xfer_add_cb (void *data, const char *signal, const char *type_data,
                             _("%s%s: cannot find available port for xfer"),
                             weechat_prefix ("error"), XFER_PLUGIN_NAME);
             close (sock);
-            if (filename2)
-                free (filename2);
             goto error;
         }
     }
@@ -1122,14 +1244,20 @@ xfer_add_cb (void *data, const char *signal, const char *type_data,
 
     /* add xfer entry and listen to socket if type is file or chat "send" */
     if (XFER_IS_FILE(type))
+    {
         ptr_xfer = xfer_new (plugin_name, plugin_id, type, protocol,
                              remote_nick, local_nick, charset_modifier,
-                             short_filename, file_size, proxy, local_addr,
-                             port, sock, filename2);
+                             short_filename, file_size, proxy,
+                             (struct sockaddr *)&addr, length, port, sock,
+                             filename2);
+    }
     else
+    {
         ptr_xfer = xfer_new (plugin_name, plugin_id, type, protocol,
                              remote_nick, local_nick, charset_modifier, NULL,
-                             0, proxy, local_addr, port, sock, NULL);
+                             0, proxy, (struct sockaddr *)&addr, length,
+                             port, sock, NULL);
+    }
 
     if (!ptr_xfer)
     {
@@ -1137,10 +1265,6 @@ xfer_add_cb (void *data, const char *signal, const char *type_data,
                         _("%s%s: error creating xfer"),
                         weechat_prefix ("error"), XFER_PLUGIN_NAME);
         close (sock);
-        if (short_filename)
-            free (short_filename);
-        if (filename2)
-            free (filename2);
         goto error;
     }
 
@@ -1148,15 +1272,18 @@ xfer_add_cb (void *data, const char *signal, const char *type_data,
     if (XFER_IS_SEND(ptr_xfer->type) && !XFER_HAS_ENDED(ptr_xfer->status))
         xfer_send_signal (ptr_xfer, "xfer_send_ready");
 
-    if (short_filename)
-        free (short_filename);
     if (filename2)
         free (filename2);
-
+    if (short_filename)
+        free (short_filename);
     weechat_infolist_reset_item_cursor (infolist);
     return WEECHAT_RC_OK;
 
 error:
+    if (filename2)
+        free (filename2);
+    if (short_filename)
+        free (short_filename);
     weechat_infolist_reset_item_cursor (infolist);
     return WEECHAT_RC_ERROR;
 }
@@ -1382,11 +1509,9 @@ xfer_add_to_infolist (struct t_infolist *infolist, struct t_xfer *xfer)
         return 0;
     if (!weechat_infolist_new_var_string (ptr_item, "proxy", xfer->proxy))
         return 0;
-    snprintf (value, sizeof (value), "%lu", xfer->local_address);
-    if (!weechat_infolist_new_var_string (ptr_item, "local_address", value))
+    if (!weechat_infolist_new_var_string (ptr_item, "local_address", xfer->local_address_str))
         return 0;
-    snprintf (value, sizeof (value), "%lu", xfer->remote_address);
-    if (!weechat_infolist_new_var_string (ptr_item, "remote_address", value))
+    if (!weechat_infolist_new_var_string (ptr_item, "remote_address", xfer->remote_address_str))
         return 0;
     if (!weechat_infolist_new_var_integer (ptr_item, "port", xfer->port))
         return 0;
@@ -1449,6 +1574,12 @@ xfer_add_to_infolist (struct t_infolist *infolist, struct t_xfer *xfer)
     snprintf (value, sizeof (value), "%llu", xfer->eta);
     if (!weechat_infolist_new_var_string (ptr_item, "eta", value))
         return 0;
+    if (!weechat_infolist_new_var_string (ptr_item, "hash_target", xfer->hash_target))
+        return 0;
+    if (!weechat_infolist_new_var_integer (ptr_item, "hash_status", xfer->hash_status))
+        return 0;
+    if (!weechat_infolist_new_var_string (ptr_item, "hash_status_string", xfer_hash_status_string[xfer->hash_status]))
+        return 0;
 
     return 1;
 }
@@ -1466,53 +1597,61 @@ xfer_print_log ()
     {
         weechat_log_printf ("");
         weechat_log_printf ("[xfer (addr:0x%lx)]", ptr_xfer);
-        weechat_log_printf ("  plugin_name . . . . : '%s'",  ptr_xfer->plugin_name);
-        weechat_log_printf ("  plugin_id . . . . . : '%s'",  ptr_xfer->plugin_id);
-        weechat_log_printf ("  type. . . . . . . . : %d (%s)",
+        weechat_log_printf ("  plugin_name . . . . . . : '%s'",  ptr_xfer->plugin_name);
+        weechat_log_printf ("  plugin_id . . . . . . . : '%s'",  ptr_xfer->plugin_id);
+        weechat_log_printf ("  type. . . . . . . . . . : %d (%s)",
                             ptr_xfer->type,
                             xfer_type_string[ptr_xfer->type]);
-        weechat_log_printf ("  protocol. . . . . . : %d (%s)",
+        weechat_log_printf ("  protocol. . . . . . . . : %d (%s)",
                             ptr_xfer->protocol,
                             xfer_protocol_string[ptr_xfer->protocol]);
-        weechat_log_printf ("  remote_nick . . . . : '%s'",  ptr_xfer->remote_nick);
-        weechat_log_printf ("  local_nick. . . . . : '%s'",  ptr_xfer->local_nick);
-        weechat_log_printf ("  charset_modifier. . : '%s'",  ptr_xfer->charset_modifier);
-        weechat_log_printf ("  filename. . . . . . : '%s'",  ptr_xfer->filename);
-        weechat_log_printf ("  size. . . . . . . . : %llu",  ptr_xfer->size);
-        weechat_log_printf ("  proxy . . . . . . . : '%s'",  ptr_xfer->proxy);
-        weechat_log_printf ("  local_address . . . : %lu",   ptr_xfer->local_address);
-        weechat_log_printf ("  remote_address. . . : %lu",   ptr_xfer->remote_address);
-        weechat_log_printf ("  port. . . . . . . . : %d",    ptr_xfer->port);
+        weechat_log_printf ("  remote_nick . . . . . . : '%s'",  ptr_xfer->remote_nick);
+        weechat_log_printf ("  local_nick. . . . . . . : '%s'",  ptr_xfer->local_nick);
+        weechat_log_printf ("  charset_modifier. . . . : '%s'",  ptr_xfer->charset_modifier);
+        weechat_log_printf ("  filename. . . . . . . . : '%s'",  ptr_xfer->filename);
+        weechat_log_printf ("  size. . . . . . . . . . : %llu",  ptr_xfer->size);
+        weechat_log_printf ("  proxy . . . . . . . . . : '%s'",  ptr_xfer->proxy);
+        weechat_log_printf ("  local_address . . . . . : 0x%lx", ptr_xfer->local_address);
+        weechat_log_printf ("  local_address_length. . : %d",    ptr_xfer->local_address_length);
+        weechat_log_printf ("  local_address_str . . . : '%s'" , ptr_xfer->local_address_str);
+        weechat_log_printf ("  remote_address. . . . . : 0x%lx", ptr_xfer->remote_address);
+        weechat_log_printf ("  remote_address_length . : %d",   ptr_xfer->remote_address_length);
+        weechat_log_printf ("  remote_address_str. . . : '%s'",  ptr_xfer->remote_address_str);
+        weechat_log_printf ("  port. . . . . . . . . . : %d",    ptr_xfer->port);
 
-        weechat_log_printf ("  status. . . . . . . : %d (%s)",
+        weechat_log_printf ("  status. . . . . . . . . : %d (%s)",
                             ptr_xfer->status,
                             xfer_status_string[ptr_xfer->status]);
-        weechat_log_printf ("  buffer. . . . . . . : 0x%lx", ptr_xfer->buffer);
-        weechat_log_printf ("  remote_nick_color . : '%s'",  ptr_xfer->remote_nick_color);
-        weechat_log_printf ("  fast_send . . . . . : %d",    ptr_xfer->fast_send);
-        weechat_log_printf ("  blocksize . . . . . : %d",    ptr_xfer->blocksize);
-        weechat_log_printf ("  start_time. . . . . : %ld",   ptr_xfer->start_time);
-        weechat_log_printf ("  start_transfer. . . : %ld",   ptr_xfer->start_transfer);
-        weechat_log_printf ("  sock. . . . . . . . : %d",    ptr_xfer->sock);
-        weechat_log_printf ("  child_pid . . . . . : %d",    ptr_xfer->child_pid);
-        weechat_log_printf ("  child_read. . . . . : %d",    ptr_xfer->child_read);
-        weechat_log_printf ("  child_write . . . . : %d",    ptr_xfer->child_write);
-        weechat_log_printf ("  hook_fd . . . . . . : 0x%lx", ptr_xfer->hook_fd);
-        weechat_log_printf ("  hook_timer. . . . . : 0x%lx", ptr_xfer->hook_timer);
-        weechat_log_printf ("  unterminated_message: '%s'",  ptr_xfer->unterminated_message);
-        weechat_log_printf ("  file. . . . . . . . : %d",    ptr_xfer->file);
-        weechat_log_printf ("  local_filename. . . : '%s'",  ptr_xfer->local_filename);
-        weechat_log_printf ("  filename_suffix . . : %d",    ptr_xfer->filename_suffix);
-        weechat_log_printf ("  pos . . . . . . . . : %llu",  ptr_xfer->pos);
-        weechat_log_printf ("  ack . . . . . . . . : %llu",  ptr_xfer->ack);
-        weechat_log_printf ("  start_resume. . . . : %llu",  ptr_xfer->start_resume);
-        weechat_log_printf ("  last_check_time . . : %ld",   ptr_xfer->last_check_time);
-        weechat_log_printf ("  last_check_pos. . . : %llu",  ptr_xfer->last_check_pos);
-        weechat_log_printf ("  last_activity . . . : %ld",   ptr_xfer->last_activity);
-        weechat_log_printf ("  bytes_per_sec . . . : %llu",  ptr_xfer->bytes_per_sec);
-        weechat_log_printf ("  eta . . . . . . . . : %llu",  ptr_xfer->eta);
-        weechat_log_printf ("  prev_xfer . . . . . : 0x%lx", ptr_xfer->prev_xfer);
-        weechat_log_printf ("  next_xfer . . . . . : 0x%lx", ptr_xfer->next_xfer);
+        weechat_log_printf ("  buffer. . . . . . . . . : 0x%lx", ptr_xfer->buffer);
+        weechat_log_printf ("  remote_nick_color . . . : '%s'",  ptr_xfer->remote_nick_color);
+        weechat_log_printf ("  fast_send . . . . . . . : %d",    ptr_xfer->fast_send);
+        weechat_log_printf ("  blocksize . . . . . . . : %d",    ptr_xfer->blocksize);
+        weechat_log_printf ("  start_time. . . . . . . : %ld",   ptr_xfer->start_time);
+        weechat_log_printf ("  start_transfer. . . . . : %ld",   ptr_xfer->start_transfer);
+        weechat_log_printf ("  sock. . . . . . . . . . : %d",    ptr_xfer->sock);
+        weechat_log_printf ("  child_pid . . . . . . . : %d",    ptr_xfer->child_pid);
+        weechat_log_printf ("  child_read. . . . . . . : %d",    ptr_xfer->child_read);
+        weechat_log_printf ("  child_write . . . . . . : %d",    ptr_xfer->child_write);
+        weechat_log_printf ("  hook_fd . . . . . . . . : 0x%lx", ptr_xfer->hook_fd);
+        weechat_log_printf ("  hook_timer. . . . . . . : 0x%lx", ptr_xfer->hook_timer);
+        weechat_log_printf ("  unterminated_message. . : '%s'",  ptr_xfer->unterminated_message);
+        weechat_log_printf ("  file. . . . . . . . . . : %d",    ptr_xfer->file);
+        weechat_log_printf ("  local_filename. . . . . : '%s'",  ptr_xfer->local_filename);
+        weechat_log_printf ("  filename_suffix . . . . : %d",    ptr_xfer->filename_suffix);
+        weechat_log_printf ("  pos . . . . . . . . . . : %llu",  ptr_xfer->pos);
+        weechat_log_printf ("  ack . . . . . . . . . . : %llu",  ptr_xfer->ack);
+        weechat_log_printf ("  start_resume. . . . . . : %llu",  ptr_xfer->start_resume);
+        weechat_log_printf ("  last_check_time . . . . : %ld",   ptr_xfer->last_check_time);
+        weechat_log_printf ("  last_check_pos. . . . . : %llu",  ptr_xfer->last_check_pos);
+        weechat_log_printf ("  last_activity . . . . . : %ld",   ptr_xfer->last_activity);
+        weechat_log_printf ("  bytes_per_sec . . . . . : %llu",  ptr_xfer->bytes_per_sec);
+        weechat_log_printf ("  eta . . . . . . . . . . : %llu",  ptr_xfer->eta);
+        weechat_log_printf ("  hash_target . . . . . . : '%s'",  ptr_xfer->hash_target);
+        weechat_log_printf ("  hash_status . . . . . . : %d (%s)",
+                            ptr_xfer->hash_status,
+                            xfer_hash_status_string[ptr_xfer->hash_status]);
+        weechat_log_printf ("  prev_xfer . . . . . . . : 0x%lx", ptr_xfer->prev_xfer);
+        weechat_log_printf ("  next_xfer . . . . . . . : 0x%lx", ptr_xfer->next_xfer);
     }
 }
 
@@ -1560,8 +1699,7 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
     if (!xfer_config_init ())
         return WEECHAT_RC_ERROR;
 
-    if (xfer_config_read () < 0)
-        return WEECHAT_RC_ERROR;
+    xfer_config_read ();
 
     xfer_create_directories ();
 
